@@ -4,7 +4,7 @@
 //! Helper functions for dispatch operations.
 
 use crate::{
-    ExpectedError, Result,
+    ExpectedError, ExtractOutputFormat, Result,
     cargo_cli::{CargoCli, CargoOptions},
     output::{OutputContext, StderrStyles},
 };
@@ -16,9 +16,13 @@ use nextest_runner::{
     cargo_config::{CargoConfigs, TargetTriple},
     errors::TargetTripleError,
     platform::{BuildPlatforms, HostPlatform, Platform, PlatformLibdir, TargetPlatform},
-    reporter::TestOutputErrorSlice,
+    reporter::{
+        TestOutputErrorSlice,
+        events::{FinalRunStats, RunStatsFailureKind},
+    },
+    run_mode::NextestRunMode,
     target_runner::{PlatformRunner, TargetRunner},
-    user_config::UserConfig,
+    user_config::{UserConfig, UserConfigLocation},
 };
 use owo_colors::OwoColorize;
 use std::io::Write;
@@ -86,8 +90,11 @@ pub(super) fn detect_build_platforms(
 }
 
 /// Loads and resolves user configuration with platform-specific overrides.
-pub(super) fn resolve_user_config(host_platform: &Platform) -> Result<UserConfig, ExpectedError> {
-    UserConfig::for_host_platform(host_platform)
+pub(super) fn resolve_user_config(
+    host_platform: &Platform,
+    location: UserConfigLocation<'_>,
+) -> Result<UserConfig, ExpectedError> {
+    UserConfig::for_host_platform(host_platform, location)
         .map_err(|e| ExpectedError::UserConfigError { err: Box::new(e) })
 }
 
@@ -187,13 +194,13 @@ pub(super) fn extract_slice_from_output<'a>(
 
 pub(super) fn display_output_slice(
     output_slice: Option<TestOutputErrorSlice<'_>>,
-    output_format: super::commands::ExtractOutputFormat,
+    output_format: ExtractOutputFormat,
 ) -> Result<()> {
     use nextest_runner::reporter::highlight_end;
     use quick_junit::XmlString;
 
     match output_format {
-        super::commands::ExtractOutputFormat::Raw => {
+        ExtractOutputFormat::Raw => {
             if let Some(kind) = output_slice
                 && let Some(out) = kind.combined_subslice()
             {
@@ -205,12 +212,12 @@ pub(super) fn display_output_slice(
                 });
             }
         }
-        super::commands::ExtractOutputFormat::JunitDescription => {
+        ExtractOutputFormat::JunitDescription => {
             if let Some(kind) = output_slice {
                 println!("{}", XmlString::new(kind.to_string()).as_str());
             }
         }
-        super::commands::ExtractOutputFormat::Highlight => {
+        ExtractOutputFormat::Highlight => {
             if let Some(kind) = output_slice
                 && let Some(out) = kind.combined_subslice()
             {
@@ -228,3 +235,36 @@ pub(super) fn display_output_slice(
     eprintln!("(no description found)");
     Ok(())
 }
+
+/// Converts final run statistics to an error, if the run failed.
+///
+/// Returns `None` if the run was successful. For `NoTestsRun`, always returns
+/// an error with `is_default: true`; callers that want custom `NoTestsBehavior`
+/// handling should check for that case separately.
+pub(super) fn final_stats_to_error(
+    stats: FinalRunStats,
+    mode: NextestRunMode,
+    rerun_available: bool,
+) -> Option<ExpectedError> {
+    match stats {
+        FinalRunStats::Success => None,
+        FinalRunStats::NoTestsRun => Some(ExpectedError::NoTestsRun {
+            mode,
+            is_default: true,
+        }),
+        FinalRunStats::Cancelled {
+            kind: RunStatsFailureKind::SetupScript,
+            ..
+        }
+        | FinalRunStats::Failed {
+            kind: RunStatsFailureKind::SetupScript,
+        } => Some(ExpectedError::setup_script_failed()),
+        FinalRunStats::Cancelled {
+            kind: RunStatsFailureKind::Test { .. },
+            ..
+        }
+        | FinalRunStats::Failed {
+            kind: RunStatsFailureKind::Test { .. },
+        } => Some(ExpectedError::test_run_failed(rerun_available)),
+    }
+}
