@@ -9,8 +9,8 @@ use nextest_metadata::NextestExitCode;
 use nextest_runner::{
     config::core::{ConfigExperimental, ToolName},
     errors::{
-        CacheDirError, PortableRecordingError, PortableRecordingReadError, RecordReadError,
-        RecordSetupError, RunIdResolutionError, RunStoreError, TestListFromSummaryError,
+        PortableRecordingError, PortableRecordingReadError, RecordReadError, RecordSetupError,
+        RunIdResolutionError, RunStoreError, StateDirError, TestListFromSummaryError,
         UserConfigError, *,
     },
     helpers::{format_interceptor_too_many_tests, plural},
@@ -139,9 +139,9 @@ pub enum ExpectedError {
         err: Box<UserConfigError>,
     },
     #[error("test filter build error")]
-    TestFilterBuilderError {
+    TestFilterBuildError {
         #[from]
-        err: TestFilterBuilderError,
+        err: TestFilterBuildError,
     },
     #[error("unknown host platform")]
     HostPlatformDetectError {
@@ -355,10 +355,10 @@ pub enum ExpectedError {
         config_file: Utf8PathBuf,
         missing: Vec<ConfigExperimental>,
     },
-    #[error("could not determine cache directory for recording")]
-    RecordCacheDirNotFound {
+    #[error("could not determine state directory for recording")]
+    RecordStateDirNotFound {
         #[source]
-        err: CacheDirError,
+        err: StateDirError,
     },
     #[error("error setting up recording")]
     RecordSetupError {
@@ -581,7 +581,7 @@ impl ExpectedError {
             | Self::RootManifestNotFound { .. }
             | Self::CargoConfigError { .. }
             | Self::UserConfigError { .. }
-            | Self::TestFilterBuilderError { .. }
+            | Self::TestFilterBuildError { .. }
             | Self::HostPlatformDetectError { .. }
             | Self::TargetTripleError { .. }
             | Self::RemapAbsoluteError { .. }
@@ -641,7 +641,7 @@ impl ExpectedError {
             | Self::ConfigExperimentalFeaturesNotEnabled { .. } => {
                 NextestExitCode::EXPERIMENTAL_FEATURE_NOT_ENABLED
             }
-            Self::RecordCacheDirNotFound { .. }
+            Self::RecordStateDirNotFound { .. }
             | Self::RecordSetupError { .. }
             | Self::RecordSessionSetupError { .. }
             | Self::RunIdResolutionError { .. }
@@ -915,7 +915,7 @@ impl ExpectedError {
                 error!("{err}");
                 err.source()
             }
-            Self::TestFilterBuilderError { err } => {
+            Self::TestFilterBuildError { err } => {
                 error!("{err}");
                 err.source()
             }
@@ -1208,8 +1208,8 @@ impl ExpectedError {
                 );
                 None
             }
-            Self::RecordCacheDirNotFound { err } => {
-                error!("could not determine cache directory for recording test runs");
+            Self::RecordStateDirNotFound { err } => {
+                error!("could not determine state directory for recording test runs");
                 Some(err as &dyn Error)
             }
             Self::RecordSetupError { err } => {
