@@ -1,15 +1,16 @@
 // Copyright (c) The nextest Contributors
 // SPDX-License-Identifier: MIT OR Apache-2.0
 
-use super::{NextestVersionDeserialize, ToolConfigFile};
+use super::{ExperimentalDeserialize, NextestVersionDeserialize, ToolConfigFile, ToolName};
 use crate::{
     config::{
         core::ConfigExperimental,
         elements::{
-            ArchiveConfig, CustomTestGroup, DefaultJunitImpl, GlobalTimeout, Inherits, JunitConfig,
-            JunitImpl, JunitSettings, LeakTimeout, MaxFail, RetryPolicy, SlowTimeout, TestGroup,
-            TestGroupConfig, TestThreads, ThreadsRequired, deserialize_fail_fast,
-            deserialize_leak_timeout, deserialize_retry_policy, deserialize_slow_timeout,
+            ArchiveConfig, BenchConfig, CustomTestGroup, DefaultBenchConfig, DefaultJunitImpl,
+            GlobalTimeout, Inherits, JunitConfig, JunitImpl, JunitSettings, LeakTimeout, MaxFail,
+            RetryPolicy, SlowTimeout, TestGroup, TestGroupConfig, TestThreads, ThreadsRequired,
+            deserialize_fail_fast, deserialize_leak_timeout, deserialize_retry_policy,
+            deserialize_slow_timeout,
         },
         overrides::{
             CompiledByProfile, CompiledData, CompiledDefaultFilter, DeserializedOverride,
@@ -30,6 +31,7 @@ use crate::{
     list::TestList,
     platform::BuildPlatforms,
     reporter::{FinalStatusLevel, StatusLevel, TestOutputDisplay},
+    run_mode::NextestRunMode,
 };
 use camino::{Utf8Path, Utf8PathBuf};
 use config::{
@@ -56,7 +58,7 @@ pub trait ConfigWarnings {
         &mut self,
         config_file: &Utf8Path,
         workspace_root: &Utf8Path,
-        tool: Option<&str>,
+        tool: Option<&ToolName>,
         unknown: &BTreeSet<String>,
     );
 
@@ -65,7 +67,7 @@ pub trait ConfigWarnings {
         &mut self,
         config_file: &Utf8Path,
         workspace_root: &Utf8Path,
-        tool: Option<&str>,
+        tool: Option<&ToolName>,
         profiles: &[&str],
     );
 
@@ -74,7 +76,7 @@ pub trait ConfigWarnings {
         &mut self,
         config_file: &Utf8Path,
         workspace_root: &Utf8Path,
-        tool: Option<&str>,
+        tool: Option<&ToolName>,
     );
 
     /// Handle warning about empty script sections with neither setup nor
@@ -83,7 +85,7 @@ pub trait ConfigWarnings {
         &mut self,
         config_file: &Utf8Path,
         workspace_root: &Utf8Path,
-        tool: Option<&str>,
+        tool: Option<&ToolName>,
         profile_name: &str,
         empty_count: usize,
     );
@@ -97,15 +99,16 @@ impl ConfigWarnings for DefaultConfigWarnings {
         &mut self,
         config_file: &Utf8Path,
         workspace_root: &Utf8Path,
-        tool: Option<&str>,
+        tool: Option<&ToolName>,
         unknown: &BTreeSet<String>,
     ) {
         let mut unknown_str = String::new();
         if unknown.len() == 1 {
             // Print this on the same line.
-            unknown_str.push(' ');
+            unknown_str.push_str("key: ");
             unknown_str.push_str(unknown.iter().next().unwrap());
         } else {
+            unknown_str.push_str("keys:\n");
             for ignored_key in unknown {
                 unknown_str.push('\n');
                 unknown_str.push_str("  - ");
@@ -114,7 +117,7 @@ impl ConfigWarnings for DefaultConfigWarnings {
         }
 
         warn!(
-            "in config file {}{}, ignoring unknown configuration keys: {unknown_str}",
+            "in config file {}{}, ignoring unknown configuration {unknown_str}",
             config_file
                 .strip_prefix(workspace_root)
                 .unwrap_or(config_file),
@@ -126,7 +129,7 @@ impl ConfigWarnings for DefaultConfigWarnings {
         &mut self,
         config_file: &Utf8Path,
         workspace_root: &Utf8Path,
-        tool: Option<&str>,
+        tool: Option<&ToolName>,
         profiles: &[&str],
     ) {
         warn!(
@@ -146,7 +149,7 @@ impl ConfigWarnings for DefaultConfigWarnings {
         &mut self,
         config_file: &Utf8Path,
         workspace_root: &Utf8Path,
-        tool: Option<&str>,
+        tool: Option<&ToolName>,
     ) {
         warn!(
             "in config file {}{}, [script.*] is deprecated and will be removed in a \
@@ -162,7 +165,7 @@ impl ConfigWarnings for DefaultConfigWarnings {
         &mut self,
         config_file: &Utf8Path,
         workspace_root: &Utf8Path,
-        tool: Option<&str>,
+        tool: Option<&ToolName>,
         profile_name: &str,
         empty_count: usize,
     ) {
@@ -443,7 +446,7 @@ impl NextestConfig {
         pcx: &ParseContext<'_>,
         workspace_root: &Utf8Path,
         config_file: &Utf8Path,
-        tool: Option<&str>,
+        tool: Option<&ToolName>,
         source: File<FileSourceFile, FileFormat>,
         compiled_out: &mut CompiledByProfile,
         experimental: &BTreeSet<ConfigExperimental>,
@@ -471,7 +474,7 @@ impl NextestConfig {
                     group
                         .as_identifier()
                         .tool_components()
-                        .is_some_and(|(tool_name, _)| tool_name == tool)
+                        .is_some_and(|(tool_name, _)| tool_name == tool.as_str())
                 } else {
                     // If a tool is not specified, it must *not* be a tool identifier.
                     !group.as_identifier().is_tool_identifier()
@@ -546,7 +549,7 @@ impl NextestConfig {
                     script
                         .as_identifier()
                         .tool_components()
-                        .is_some_and(|(tool_name, _)| tool_name == tool)
+                        .is_some_and(|(tool_name, _)| tool_name == tool.as_str())
                 } else {
                     // If a tool is not specified, it must *not* be a tool identifier.
                     !script.as_identifier().is_tool_identifier()
@@ -908,12 +911,76 @@ pub struct EarlyProfile<'cfg> {
     pub(in crate::config) compiled_data: CompiledData<PreBuildPlatform>,
 }
 
+/// These macros return a specific config field from a profile, checking in
+/// order: custom profile, inheritance chain, then default profile.
+macro_rules! profile_field {
+    ($eval_prof:ident.$field:ident) => {
+        $eval_prof
+            .custom_profile
+            .iter()
+            .chain($eval_prof.inheritance_chain.iter())
+            .find_map(|p| p.$field)
+            .unwrap_or($eval_prof.default_profile.$field)
+    };
+    ($eval_prof:ident.$nested:ident.$field:ident) => {
+        $eval_prof
+            .custom_profile
+            .iter()
+            .chain($eval_prof.inheritance_chain.iter())
+            .find_map(|p| p.$nested.$field)
+            .unwrap_or($eval_prof.default_profile.$nested.$field)
+    };
+    // Variant for method calls with arguments.
+    ($eval_prof:ident.$method:ident($($arg:expr),*)) => {
+        $eval_prof
+            .custom_profile
+            .iter()
+            .chain($eval_prof.inheritance_chain.iter())
+            .find_map(|p| p.$method($($arg),*))
+            .unwrap_or_else(|| $eval_prof.default_profile.$method($($arg),*))
+    };
+}
+macro_rules! profile_field_from_ref {
+    ($eval_prof:ident.$field:ident.$ref_func:ident()) => {
+        $eval_prof
+            .custom_profile
+            .iter()
+            .chain($eval_prof.inheritance_chain.iter())
+            .find_map(|p| p.$field.$ref_func())
+            .unwrap_or(&$eval_prof.default_profile.$field)
+    };
+    ($eval_prof:ident.$nested:ident.$field:ident.$ref_func:ident()) => {
+        $eval_prof
+            .custom_profile
+            .iter()
+            .chain($eval_prof.inheritance_chain.iter())
+            .find_map(|p| p.$nested.$field.$ref_func())
+            .unwrap_or(&$eval_prof.default_profile.$nested.$field)
+    };
+}
+// Variant for fields where both custom and default are Option.
+macro_rules! profile_field_optional {
+    ($eval_prof:ident.$nested:ident.$field:ident.$ref_func:ident()) => {
+        $eval_prof
+            .custom_profile
+            .iter()
+            .chain($eval_prof.inheritance_chain.iter())
+            .find_map(|p| p.$nested.$field.$ref_func())
+            .or($eval_prof.default_profile.$nested.$field.$ref_func())
+    };
+}
+
 impl<'cfg> EarlyProfile<'cfg> {
     /// Returns the absolute profile-specific store directory.
     pub fn store_dir(&self) -> &Utf8Path {
         &self.store_dir
     }
 
+    /// Returns true if JUnit XML output is configured for this profile.
+    pub fn has_junit(&self) -> bool {
+        profile_field_optional!(self.junit.path.as_deref()).is_some()
+    }
+
     /// Returns the global test group configuration.
     pub fn test_group_config(&self) -> &'cfg BTreeMap<CustomTestGroup, TestGroupConfig> {
         self.test_groups
@@ -980,56 +1047,6 @@ pub struct EvaluatableProfile<'cfg> {
     resolved_default_filter: CompiledDefaultFilter,
 }
 
-/// These macros return a specific config field from an EvaluatableProfile,
-/// checking in order: custom profile, inheritance chain, then default profile.
-macro_rules! profile_field {
-    ($eval_prof:ident.$field:ident) => {
-        $eval_prof
-            .custom_profile
-            .iter()
-            .chain($eval_prof.inheritance_chain.iter())
-            .find_map(|p| p.$field)
-            .unwrap_or($eval_prof.default_profile.$field)
-    };
-    ($eval_prof:ident.$nested:ident.$field:ident) => {
-        $eval_prof
-            .custom_profile
-            .iter()
-            .chain($eval_prof.inheritance_chain.iter())
-            .find_map(|p| p.$nested.$field)
-            .unwrap_or($eval_prof.default_profile.$nested.$field)
-    };
-}
-macro_rules! profile_field_from_ref {
-    ($eval_prof:ident.$field:ident.$ref_func:ident()) => {
-        $eval_prof
-            .custom_profile
-            .iter()
-            .chain($eval_prof.inheritance_chain.iter())
-            .find_map(|p| p.$field.$ref_func())
-            .unwrap_or(&$eval_prof.default_profile.$field)
-    };
-    ($eval_prof:ident.$nested:ident.$field:ident.$ref_func:ident()) => {
-        $eval_prof
-            .custom_profile
-            .iter()
-            .chain($eval_prof.inheritance_chain.iter())
-            .find_map(|p| p.$nested.$field.$ref_func())
-            .unwrap_or(&$eval_prof.default_profile.$nested.$field)
-    };
-}
-// Variant for fields where both custom and default are Option.
-macro_rules! profile_field_optional {
-    ($eval_prof:ident.$nested:ident.$field:ident.$ref_func:ident()) => {
-        $eval_prof
-            .custom_profile
-            .iter()
-            .chain($eval_prof.inheritance_chain.iter())
-            .find_map(|p| p.$nested.$field.$ref_func())
-            .or($eval_prof.default_profile.$nested.$field.$ref_func())
-    };
-}
-
 impl<'cfg> EvaluatableProfile<'cfg> {
     /// Returns the name of the profile.
     pub fn name(&self) -> &str {
@@ -1084,13 +1101,13 @@ impl<'cfg> EvaluatableProfile<'cfg> {
     }
 
     /// Returns the time after which tests are treated as slow for this profile.
-    pub fn slow_timeout(&self) -> SlowTimeout {
-        profile_field!(self.slow_timeout)
+    pub fn slow_timeout(&self, run_mode: NextestRunMode) -> SlowTimeout {
+        profile_field!(self.slow_timeout(run_mode))
     }
 
     /// Returns the time after which we should stop running tests.
-    pub fn global_timeout(&self) -> GlobalTimeout {
-        profile_field!(self.global_timeout)
+    pub fn global_timeout(&self, run_mode: NextestRunMode) -> GlobalTimeout {
+        profile_field!(self.global_timeout(run_mode))
     }
 
     /// Returns the time after which a child process that hasn't closed its handles is marked as
@@ -1140,16 +1157,21 @@ impl<'cfg> EvaluatableProfile<'cfg> {
     }
 
     /// Returns settings for individual tests.
-    pub fn settings_for(&self, query: &TestQuery<'_>) -> TestSettings<'_> {
-        TestSettings::new(self, query)
+    pub fn settings_for(
+        &self,
+        run_mode: NextestRunMode,
+        query: &TestQuery<'_>,
+    ) -> TestSettings<'_> {
+        TestSettings::new(self, run_mode, query)
     }
 
     /// Returns override settings for individual tests, with sources attached.
     pub(crate) fn settings_with_source_for(
         &self,
+        run_mode: NextestRunMode,
         query: &TestQuery<'_>,
     ) -> TestSettings<'_, SettingSource<'_>> {
-        TestSettings::new(self, query)
+        TestSettings::new(self, run_mode, query)
     }
 
     /// Returns the JUnit configuration for this profile.
@@ -1425,7 +1447,7 @@ struct NextestConfigDeserialize {
     nextest_version: Option<NextestVersionDeserialize>,
     #[expect(unused)]
     #[serde(default)]
-    experimental: BTreeSet<String>,
+    experimental: ExperimentalDeserialize,
 
     #[serde(default)]
     test_groups: BTreeMap<CustomTestGroup, TestGroupConfig>,
@@ -1491,6 +1513,7 @@ pub(in crate::config) struct DefaultProfileImpl {
     scripts: Vec<DeserializedProfileScriptConfig>,
     junit: DefaultJunitImpl,
     archive: ArchiveConfig,
+    bench: DefaultBenchConfig,
     inherits: Inherits,
 }
 
@@ -1536,6 +1559,9 @@ impl DefaultProfileImpl {
             scripts: p.scripts,
             junit: DefaultJunitImpl::for_default_profile(p.junit),
             archive: p.archive.expect("archive present in default profile"),
+            bench: DefaultBenchConfig::for_default_profile(
+                p.bench.expect("bench present in default profile"),
+            ),
             inherits: Inherits::new(p.inherits),
         }
     }
@@ -1555,6 +1581,20 @@ impl DefaultProfileImpl {
     pub(in crate::config) fn setup_scripts(&self) -> &[DeserializedProfileScriptConfig] {
         &self.scripts
     }
+
+    pub(in crate::config) fn slow_timeout(&self, run_mode: NextestRunMode) -> SlowTimeout {
+        match run_mode {
+            NextestRunMode::Test => self.slow_timeout,
+            NextestRunMode::Benchmark => self.bench.slow_timeout,
+        }
+    }
+
+    pub(in crate::config) fn global_timeout(&self, run_mode: NextestRunMode) -> GlobalTimeout {
+        match run_mode {
+            NextestRunMode::Test => self.global_timeout,
+            NextestRunMode::Benchmark => self.bench.global_timeout,
+        }
+    }
 }
 
 #[derive(Clone, Debug, Deserialize)]
@@ -1600,6 +1640,8 @@ pub(in crate::config) struct CustomProfileImpl {
     #[serde(default)]
     archive: Option<ArchiveConfig>,
     #[serde(default)]
+    bench: Option<BenchConfig>,
+    #[serde(default)]
     inherits: Option<String>,
 }
 
@@ -1613,6 +1655,23 @@ impl CustomProfileImpl {
         self.default_filter.as_deref()
     }
 
+    pub(in crate::config) fn slow_timeout(&self, run_mode: NextestRunMode) -> Option<SlowTimeout> {
+        match run_mode {
+            NextestRunMode::Test => self.slow_timeout,
+            NextestRunMode::Benchmark => self.bench.as_ref().and_then(|b| b.slow_timeout),
+        }
+    }
+
+    pub(in crate::config) fn global_timeout(
+        &self,
+        run_mode: NextestRunMode,
+    ) -> Option<GlobalTimeout> {
+        match run_mode {
+            NextestRunMode::Test => self.global_timeout,
+            NextestRunMode::Benchmark => self.bench.as_ref().and_then(|b| b.global_timeout),
+        }
+    }
+
     pub(in crate::config) fn inherits(&self) -> Option<&str> {
         self.inherits.as_deref()
     }
@@ -1633,6 +1692,10 @@ mod tests {
     use camino_tempfile::tempdir;
     use iddqd::{IdHashItem, IdHashMap, id_hash_map, id_upcast};
 
+    fn tool_name(s: &str) -> ToolName {
+        ToolName::new(s.into()).unwrap()
+    }
+
     /// Test implementation of ConfigWarnings that collects warnings for testing.
     #[derive(Default)]
     struct TestConfigWarnings {
@@ -1647,12 +1710,12 @@ mod tests {
             &mut self,
             config_file: &Utf8Path,
             _workspace_root: &Utf8Path,
-            tool: Option<&str>,
+            tool: Option<&ToolName>,
             unknown: &BTreeSet<String>,
         ) {
             self.unknown_keys
                 .insert_unique(UnknownKeys {
-                    tool: tool.map(|s| s.to_owned()),
+                    tool: tool.cloned(),
                     config_file: config_file.to_owned(),
                     keys: unknown.clone(),
                 })
@@ -1663,12 +1726,12 @@ mod tests {
             &mut self,
             config_file: &Utf8Path,
             _workspace_root: &Utf8Path,
-            tool: Option<&str>,
+            tool: Option<&ToolName>,
             profiles: &[&str],
         ) {
             self.reserved_profiles
                 .insert_unique(ReservedProfiles {
-                    tool: tool.map(|s| s.to_owned()),
+                    tool: tool.cloned(),
                     config_file: config_file.to_owned(),
                     profiles: profiles.iter().map(|&s| s.to_owned()).collect(),
                 })
@@ -1679,13 +1742,13 @@ mod tests {
             &mut self,
             config_file: &Utf8Path,
             _workspace_root: &Utf8Path,
-            tool: Option<&str>,
+            tool: Option<&ToolName>,
             profile_name: &str,
             empty_count: usize,
         ) {
             self.empty_script_warnings
                 .insert_unique(EmptyScriptSections {
-                    tool: tool.map(|s| s.to_owned()),
+                    tool: tool.cloned(),
                     config_file: config_file.to_owned(),
                     profile_name: profile_name.to_owned(),
                     empty_count,
@@ -1697,11 +1760,11 @@ mod tests {
             &mut self,
             config_file: &Utf8Path,
             _workspace_root: &Utf8Path,
-            tool: Option<&str>,
+            tool: Option<&ToolName>,
         ) {
             self.deprecated_scripts
                 .insert_unique(DeprecatedScripts {
-                    tool: tool.map(|s| s.to_owned()),
+                    tool: tool.cloned(),
                     config_file: config_file.to_owned(),
                 })
                 .unwrap();
@@ -1710,58 +1773,58 @@ mod tests {
 
     #[derive(Clone, Debug, PartialEq, Eq, PartialOrd, Ord)]
     struct UnknownKeys {
-        tool: Option<String>,
+        tool: Option<ToolName>,
         config_file: Utf8PathBuf,
         keys: BTreeSet<String>,
     }
 
     impl IdHashItem for UnknownKeys {
-        type Key<'a> = Option<&'a str>;
+        type Key<'a> = Option<&'a ToolName>;
         fn key(&self) -> Self::Key<'_> {
-            self.tool.as_deref()
+            self.tool.as_ref()
         }
         id_upcast!();
     }
 
     #[derive(Clone, Debug, PartialEq, Eq, PartialOrd, Ord)]
     struct ReservedProfiles {
-        tool: Option<String>,
+        tool: Option<ToolName>,
         config_file: Utf8PathBuf,
         profiles: Vec<String>,
     }
 
     impl IdHashItem for ReservedProfiles {
-        type Key<'a> = Option<&'a str>;
+        type Key<'a> = Option<&'a ToolName>;
         fn key(&self) -> Self::Key<'_> {
-            self.tool.as_deref()
+            self.tool.as_ref()
         }
         id_upcast!();
     }
 
     #[derive(Clone, Debug, PartialEq, Eq, PartialOrd, Ord)]
     struct DeprecatedScripts {
-        tool: Option<String>,
+        tool: Option<ToolName>,
         config_file: Utf8PathBuf,
     }
 
     impl IdHashItem for DeprecatedScripts {
-        type Key<'a> = Option<&'a str>;
+        type Key<'a> = Option<&'a ToolName>;
         fn key(&self) -> Self::Key<'_> {
-            self.tool.as_deref()
+            self.tool.as_ref()
         }
         id_upcast!();
     }
 
     #[derive(Clone, Debug, PartialEq, Eq, PartialOrd, Ord)]
     struct EmptyScriptSections {
-        tool: Option<String>,
+        tool: Option<ToolName>,
         config_file: Utf8PathBuf,
         profile_name: String,
         empty_count: usize,
     }
 
     impl IdHashItem for EmptyScriptSections {
-        type Key<'a> = (&'a Option<String>, &'a str);
+        type Key<'a> = (&'a Option<ToolName>, &'a str);
         fn key(&self) -> Self::Key<'_> {
             (&self.tool, &self.profile_name)
         }
@@ -1830,7 +1893,7 @@ mod tests {
             &pcx,
             None,
             &[ToolConfigFile {
-                tool: "my-tool".to_owned(),
+                tool: tool_name("my-tool"),
                 config_file: tool_path.clone(),
             }][..],
             &Default::default(),
@@ -1857,7 +1920,7 @@ mod tests {
                     }
                 },
                 UnknownKeys {
-                    tool: Some("my-tool".to_owned()),
+                    tool: Some(tool_name("my-tool")),
                     config_file: tool_path.clone(),
                     keys: maplit::btreeset! {
                         "store.ignored4".to_owned(),
@@ -1876,7 +1939,7 @@ mod tests {
                     profiles: vec!["default-foo".to_owned()],
                 },
                 ReservedProfiles {
-                    tool: Some("my-tool".to_owned()),
+                    tool: Some(tool_name("my-tool")),
                     config_file: tool_path,
                     profiles: vec!["default-bar".to_owned()],
                 }
@@ -1954,7 +2017,7 @@ mod tests {
             &pcx,
             None,
             &[ToolConfigFile {
-                tool: "tool".to_owned(),
+                tool: tool_name("tool"),
                 config_file: tool_path.clone(),
             }][..],
             &experimental,
@@ -1978,7 +2041,7 @@ mod tests {
                     empty_count: 2,
                 },
                 EmptyScriptSections {
-                    tool: Some("tool".to_owned()),
+                    tool: Some(tool_name("tool")),
                     config_file: tool_path,
                     profile_name: "tool".to_owned(),
                     empty_count: 1,
@@ -2017,7 +2080,7 @@ mod tests {
             &pcx,
             None,
             &[ToolConfigFile {
-                tool: "my-tool".to_owned(),
+                tool: tool_name("my-tool"),
                 config_file: tool_path.clone(),
             }],
             &maplit::btreeset! {ConfigExperimental::SetupScripts},
@@ -2033,7 +2096,7 @@ mod tests {
                     config_file: graph.workspace().root().join(".config/nextest.toml"),
                 },
                 DeprecatedScripts {
-                    tool: Some("my-tool".to_owned()),
+                    tool: Some(tool_name("my-tool")),
                     config_file: tool_path,
                 }
             }
