@@ -19,9 +19,11 @@ pub mod helpers;
 pub mod indenter;
 pub mod input;
 pub mod list;
+pub mod output_spec;
 pub mod pager;
 pub mod partition;
 pub mod platform;
+pub mod record;
 pub mod redact;
 pub mod reporter;
 pub mod reuse_build;
