@@ -10,7 +10,8 @@ use crate::{
     errors::RecordReadError,
     list::OwnedTestInstanceId,
     record::{
-        CoreEventKind, OutputEventKind, RecordReader, TestEventKindSummary,
+        CoreEventKind, OutputEventKind, PortableRecording, RecordReader, StoreReader,
+        TestEventKindSummary,
         format::{RerunInfo, RerunRootInfo, RerunTestSuiteInfo},
     },
 };
@@ -20,7 +21,10 @@ use nextest_metadata::{
     TestListSummary,
 };
 use quick_junit::ReportUuid;
-use std::collections::{BTreeSet, HashMap};
+use std::{
+    borrow::Borrow,
+    collections::{BTreeSet, HashMap},
+};
 
 /// Trait abstracting over test list access for rerun computation.
 ///
@@ -263,6 +267,39 @@ impl ComputedRerunInfo {
         ))
     }
 
+    /// Computes outstanding tests from a portable recording.
+    ///
+    /// If the archive is itself a rerun, also returns information about the
+    /// root of the chain.
+    pub fn compute_from_archive(
+        archive: &mut PortableRecording,
+    ) -> Result<(Self, Option<RerunRootInfo>), RecordReadError> {
+        let mut store = archive
+            .open_store()
+            .map_err(RecordReadError::PortableRecording)?;
+        let rerun_info = store.read_rerun_info()?;
+        let test_list = store.read_test_list()?;
+        // No need to load dictionaries since we're not using them.
+
+        // Read events from the archive's run log.
+        let run_log = archive
+            .read_run_log()
+            .map_err(RecordReadError::PortableRecording)?;
+        let outcomes = collect_from_events(run_log.events()?.map(|r| r.map(|e| e.kind)))?;
+
+        let prev_test_suites = rerun_info.as_ref().map(|info| &info.test_suites);
+        let new_test_suites = compute_outstanding_pure(prev_test_suites, &test_list, &outcomes);
+
+        let root_info = rerun_info.map(|info| info.root_info);
+
+        Ok((
+            Self {
+                test_suites: new_test_suites,
+            },
+            root_info,
+        ))
+    }
+
     /// Consumes self, converting to a [`RerunInfo`] for storage.
     pub fn into_rerun_info(self, parent_run_id: ReportUuid, root_info: RerunRootInfo) -> RerunInfo {
         RerunInfo {
@@ -372,28 +409,28 @@ impl TestEventOutcomes {
     /// (had any event: started, finished, or skipped).
     fn collect(reader: &mut RecordReader) -> Result<Self, RecordReadError> {
         reader.load_dictionaries()?;
-
-        let events: Vec<_> = reader.events()?.collect::<Result<Vec<_>, _>>()?;
-        let outcomes = collect_from_events(events.iter().map(|e| &e.kind));
+        let outcomes = collect_from_events(reader.events()?.map(|r| r.map(|e| e.kind)))?;
 
         Ok(Self { outcomes })
     }
 }
 
-/// Collects test outcomes from an iterator of events.
+/// Collects test outcomes from a fallible iterator of events.
 ///
 /// This helper exists to make the event processing logic testable without
-/// requiring a full `RecordReader`.
-fn collect_from_events<'a, O>(
-    events: impl Iterator<Item = &'a TestEventKindSummary<O>>,
-) -> HashMap<OwnedTestInstanceId, TestOutcome>
+/// requiring a full `RecordReader`. It accepts a fallible iterator to enable
+/// streaming without in-memory buffering.
+fn collect_from_events<K, S: crate::output_spec::OutputSpec, E>(
+    events: impl Iterator<Item = Result<K, E>>,
+) -> Result<HashMap<OwnedTestInstanceId, TestOutcome>, E>
 where
-    O: 'a,
+    K: Borrow<TestEventKindSummary<S>>,
 {
     let mut outcomes = HashMap::new();
 
-    for kind in events {
-        match kind {
+    for kind_result in events {
+        let kind = kind_result?;
+        match kind.borrow() {
             TestEventKindSummary::Output(OutputEventKind::TestFinished {
                 test_instance,
                 run_statuses,
@@ -433,20 +470,22 @@ where
         }
     }
 
-    outcomes
+    Ok(outcomes)
 }
 
 #[cfg(test)]
 mod tests {
     use super::*;
     use crate::{
-        record::{OutputEventKind, StressIndexSummary, TestEventKindSummary},
+        output_spec::RecordingSpec,
+        record::{
+            OutputEventKind, StressIndexSummary, TestEventKindSummary, ZipStoreOutputDescription,
+        },
         reporter::{
             TestOutputDisplay,
             events::{
-                ChildExecutionOutputDescription, ChildOutputDescription, ExecuteStatus,
-                ExecutionResultDescription, ExecutionStatuses, FailureDescription, RetryData,
-                RunStats,
+                ChildExecutionOutputDescription, ExecuteStatus, ExecutionResultDescription,
+                ExecutionStatuses, FailureDescription, RetryData, RunStats,
             },
         },
     };
@@ -454,6 +493,7 @@ mod tests {
     use proptest::prelude::*;
     use std::{
         collections::{BTreeMap, btree_map},
+        convert::Infallible,
         num::NonZero,
         sync::OnceLock,
         time::Duration,
@@ -699,6 +739,164 @@ mod tests {
         );
     }
 
+    // ---
+    // Spec property verification
+    // ---
+    //
+    // These tests verify properties of the decision table itself (not the
+    // implementation). Since the (sub)domain is finite, we enumerate all cases.
+
+    /// All possible previous states.
+    const ALL_PREV_STATUSES: [PrevStatus; 3] = [
+        PrevStatus::Passing,
+        PrevStatus::Outstanding,
+        PrevStatus::Unknown,
+    ];
+
+    /// All possible outcomes (including None = not seen).
+    fn all_outcomes() -> [Option<TestOutcome>; 5] {
+        [
+            None,
+            Some(TestOutcome::Passed),
+            Some(TestOutcome::Failed),
+            Some(TestOutcome::Skipped(TestOutcomeSkipped::Rerun)),
+            Some(TestOutcome::Skipped(TestOutcomeSkipped::Explicit)),
+        ]
+    }
+
+    /// All HasMatch filter results (test is in the list).
+    fn all_in_list_filter_results() -> Vec<FilterMatchResult> {
+        let mut results = vec![FilterMatchResult::HasMatch(FilterMatch::Matches)];
+        for &reason in MismatchReason::ALL_VARIANTS {
+            results.push(FilterMatchResult::HasMatch(FilterMatch::Mismatch {
+                reason,
+            }));
+        }
+        results
+    }
+
+    /// Spec property: Passing tests stay Passing under non-regressing conditions.
+    ///
+    /// A test that was Passing remains Passing if:
+    /// - It's still in the test list (any HasMatch variant)
+    /// - Its outcome is non-regressing (Passed, Skipped(Rerun), or Skipped(Explicit))
+    ///
+    /// Verified exhaustively: 8 filter variants × 3 outcomes = 24 cases.
+    #[test]
+    fn spec_property_passing_monotonicity() {
+        let non_regressing_outcomes = [
+            Some(TestOutcome::Passed),
+            Some(TestOutcome::Skipped(TestOutcomeSkipped::Rerun)),
+            Some(TestOutcome::Skipped(TestOutcomeSkipped::Explicit)),
+        ];
+
+        for filter in all_in_list_filter_results() {
+            for outcome in non_regressing_outcomes {
+                let decision = decide_test_outcome(PrevStatus::Passing, filter, outcome);
+                assert_eq!(
+                    decision,
+                    Decision::Passing,
+                    "monotonicity violated: Passing + {:?} + {:?} -> {:?}",
+                    filter,
+                    outcome,
+                    decision
+                );
+            }
+        }
+    }
+
+    /// Spec property: Outstanding tests become Passing when they pass.
+    ///
+    /// This is the convergence property: the only way out of Outstanding is to
+    /// pass.
+    #[test]
+    fn spec_property_outstanding_to_passing_on_pass() {
+        let passing_outcomes = [
+            Some(TestOutcome::Passed),
+            Some(TestOutcome::Skipped(TestOutcomeSkipped::Rerun)),
+        ];
+
+        for outcome in passing_outcomes {
+            let decision = decide_test_outcome(
+                PrevStatus::Outstanding,
+                FilterMatchResult::HasMatch(FilterMatch::Matches),
+                outcome,
+            );
+            assert_eq!(
+                decision,
+                Decision::Passing,
+                "convergence violated: Outstanding + Matches + {:?} -> {:?}",
+                outcome,
+                decision
+            );
+        }
+    }
+
+    /// Spec property: Failed or not-seen tests become Outstanding.
+    ///
+    /// If a test matches the filter but fails or isn't seen, it's outstanding.
+    #[test]
+    fn spec_property_failed_becomes_outstanding() {
+        let failing_outcomes = [None, Some(TestOutcome::Failed)];
+
+        for prev in ALL_PREV_STATUSES {
+            for outcome in failing_outcomes {
+                let decision = decide_test_outcome(
+                    prev,
+                    FilterMatchResult::HasMatch(FilterMatch::Matches),
+                    outcome,
+                );
+                assert_eq!(
+                    decision,
+                    Decision::Outstanding,
+                    "FAILED->OUTSTANDING VIOLATED: {:?} + Matches + {:?} -> {:?}",
+                    prev,
+                    outcome,
+                    decision
+                );
+            }
+        }
+    }
+
+    /// Spec property: Carry-forward preserves Outstanding but drops Passing for
+    /// tests not in the list.
+    ///
+    /// When a listed binary no longer contains a test (TestNotInList),
+    /// Outstanding is preserved but Passing is dropped (becomes NotTracked).
+    /// This ensures tests that disappear and reappear are re-run.
+    #[test]
+    fn spec_property_test_not_in_list_behavior() {
+        for outcome in all_outcomes() {
+            // Outstanding is preserved.
+            assert_eq!(
+                decide_test_outcome(
+                    PrevStatus::Outstanding,
+                    FilterMatchResult::TestNotInList,
+                    outcome
+                ),
+                Decision::Outstanding,
+            );
+            // Passing is dropped.
+            assert_eq!(
+                decide_test_outcome(
+                    PrevStatus::Passing,
+                    FilterMatchResult::TestNotInList,
+                    outcome
+                ),
+                Decision::NotTracked,
+            );
+            // Unknown stays untracked.
+            assert_eq!(
+                decide_test_outcome(
+                    PrevStatus::Unknown,
+                    FilterMatchResult::TestNotInList,
+                    outcome
+                ),
+                Decision::NotTracked,
+            );
+        }
+    }
+
     // ---
     // Model types
     // ---
@@ -1248,13 +1446,13 @@ mod tests {
 
     /// Creates a `TestFinished` event for testing.
     ///
-    /// Uses `()` as the output type since we don't need actual output data for
-    /// these tests.
+    /// Uses `RecordingSpec` as the output spec since we don't need actual output
+    /// data for these tests (all outputs are `None`).
     fn make_test_finished(
         test_instance: OwnedTestInstanceId,
         stress_index: Option<(u32, Option<u32>)>,
         passed: bool,
-    ) -> TestEventKindSummary<()> {
+    ) -> TestEventKindSummary<RecordingSpec> {
         let result = if passed {
             ExecutionResultDescription::Pass
         } else {
@@ -1271,7 +1469,7 @@ mod tests {
             },
             output: ChildExecutionOutputDescription::Output {
                 result: Some(result.clone()),
-                output: ChildOutputDescription::Split {
+                output: ZipStoreOutputDescription::Split {
                     stdout: None,
                     stderr: None,
                 },
@@ -1369,7 +1567,7 @@ mod tests {
             make_test_finished(test_regular_fail.clone(), None, false),
         ];
 
-        let outcomes = collect_from_events(events.iter());
+        let outcomes = collect_from_events(events.iter().map(Ok::<_, Infallible>)).unwrap();
 
         assert_eq!(
             outcomes.get(&test_pass_fail_pass),
@@ -1428,7 +1626,7 @@ mod tests {
             make_test_finished(test_b.clone(), Some((1, Some(2))), false),
         ];
 
-        let outcomes = collect_from_events(events.iter());
+        let outcomes = collect_from_events(events.iter().map(Ok::<_, Infallible>)).unwrap();
 
         assert_eq!(
             outcomes.get(&test_a),
