@@ -1,12 +1,12 @@
 // Copyright (c) The nextest Contributors
 // SPDX-License-Identifier: MIT OR Apache-2.0
 
-use super::events::{AbortStatus, ExecutionResult, FailureStatus, UnitKind};
-use crate::{
-    errors::{ChildError, ChildStartError, ErrorList},
-    helpers::display_abort_status,
-    test_output::{ChildExecutionOutput, ChildOutput},
+use super::events::{
+    AbortDescription, ChildErrorDescription, ChildExecutionOutputDescription,
+    ChildOutputDescription, ChildStartErrorDescription, ExecutionResultDescription,
+    FailureDescription, UnitKind,
 };
+use crate::{errors::ErrorList, output_spec::LiveSpec};
 use bstr::ByteSlice;
 use regex::bytes::{Regex, RegexBuilder};
 use std::{fmt, sync::LazyLock};
@@ -17,25 +17,25 @@ use thiserror::Error;
 #[derive(Clone, Debug)]
 pub struct UnitErrorDescription<'a> {
     kind: UnitKind,
-    start_error: Option<&'a ChildStartError>,
-    output_errors: Option<&'a ErrorList<ChildError>>,
+    start_error: Option<&'a ChildStartErrorDescription>,
+    output_errors: Option<&'a ErrorList<ChildErrorDescription>>,
     abort: Option<UnitAbortDescription>,
     output_slice: Option<TestOutputErrorSlice<'a>>,
 }
 
 impl<'a> UnitErrorDescription<'a> {
     /// Adds the execution output of a child process to the description.
-    pub fn new(kind: UnitKind, output: &'a ChildExecutionOutput) -> Self {
+    pub fn new(kind: UnitKind, output: &'a ChildExecutionOutputDescription<LiveSpec>) -> Self {
         let mut start_error = None;
         let mut output_errors = None;
         let mut abort = None;
         let mut output_slice = None;
 
         match output {
-            ChildExecutionOutput::StartError(error) => {
+            ChildExecutionOutputDescription::StartError(error) => {
                 start_error = Some(error);
             }
-            ChildExecutionOutput::Output {
+            ChildExecutionOutputDescription::Output {
                 result,
                 output,
                 errors,
@@ -46,28 +46,36 @@ impl<'a> UnitErrorDescription<'a> {
                         match output {
                             // Scanning the output for the most relevant slice
                             // only makes sense for completed tests.
-                            ChildOutput::Split(output) => {
+                            ChildOutputDescription::Split { stdout, stderr } => {
                                 output_slice = TestOutputErrorSlice::heuristic_extract(
-                                    output.stdout.as_ref().map(|x| x.buf.as_ref()),
-                                    output.stderr.as_ref().map(|x| x.buf.as_ref()),
+                                    stdout.as_ref().map(|x| x.buf.as_ref()),
+                                    stderr.as_ref().map(|x| x.buf.as_ref()),
                                 );
                             }
-                            ChildOutput::Combined { output } => {
+                            ChildOutputDescription::Combined { output } => {
                                 output_slice = TestOutputErrorSlice::heuristic_extract(
                                     Some(output.buf.as_ref()),
                                     Some(output.buf.as_ref()),
                                 );
                             }
+                            ChildOutputDescription::NotLoaded => {
+                                unreachable!(
+                                    "attempted to extract errors from output that was not loaded \
+                                     (the OutputLoadDecider should have returned Load for this \
+                                     event)"
+                                );
+                            }
                         }
                     }
 
-                    if let ExecutionResult::Fail {
-                        failure_status: FailureStatus::Abort(status),
+                    if let ExecutionResultDescription::Fail {
+                        failure: FailureDescription::Abort { abort: status },
                         leaked,
+                        ..
                     } = result
                     {
                         abort = Some(UnitAbortDescription {
-                            status: *status,
+                            description: status.clone(),
                             leaked: *leaked,
                         });
                     }
@@ -120,12 +128,10 @@ impl<'a> UnitErrorDescription<'a> {
 
     fn exec_fail_errors(&self) -> impl Iterator<Item = &dyn std::error::Error> {
         self.start_error
-            .as_ref()
             .map(|error| error as &dyn std::error::Error)
             .into_iter()
             .chain(
                 self.output_errors
-                    .as_ref()
                     .into_iter()
                     .flat_map(|errors| errors.iter().map(|error| error as &dyn std::error::Error)),
             )
@@ -144,15 +150,15 @@ impl<'a> UnitErrorDescription<'a> {
     }
 }
 
-#[derive(Clone, Copy, Debug, Error)]
+#[derive(Clone, Debug, Error)]
 struct UnitAbortDescription {
-    status: AbortStatus,
+    description: AbortDescription,
     leaked: bool,
 }
 
 impl fmt::Display for UnitAbortDescription {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        write!(f, "process {}", display_abort_status(self.status))?;
+        write!(f, "process {}", self.description)?;
         if self.leaked {
             write!(f, ", and also leaked handles")?;
         }
