@@ -7,13 +7,16 @@
 //! reporter. The root structure for all events is [`TestEvent`].
 
 use super::{FinalStatusLevel, StatusLevel, TestOutputDisplay};
+#[cfg(test)]
+use crate::output_spec::ArbitraryOutputSpec;
 use crate::{
     config::{
         elements::{LeakTimeoutResult, SlowTimeoutResult},
         scripts::ScriptId,
     },
     errors::{ChildError, ChildFdError, ChildStartError, ErrorList},
-    list::{TestInstanceId, TestList},
+    list::{OwnedTestInstanceId, TestInstanceId, TestList},
+    output_spec::{LiveSpec, OutputSpec, SerializableOutputSpec},
     runner::{StressCondition, StressCount},
     test_output::{ChildExecutionOutput, ChildOutput, ChildSingleOutput},
 };
@@ -106,7 +109,7 @@ pub enum TestEventKind<'a> {
         program: String,
 
         /// The arguments to the program.
-        args: &'a [String],
+        args: Vec<String>,
 
         /// True if some output from the setup script is being passed through.
         no_capture: bool,
@@ -124,7 +127,7 @@ pub enum TestEventKind<'a> {
         program: String,
 
         /// The arguments to the program.
-        args: &'a [String],
+        args: Vec<String>,
 
         /// The amount of time elapsed since the start of execution.
         elapsed: Duration,
@@ -151,7 +154,7 @@ pub enum TestEventKind<'a> {
         program: String,
 
         /// The arguments to the program.
-        args: &'a [String],
+        args: Vec<String>,
 
         /// Whether the JUnit report should store success output for this script.
         junit_store_success_output: bool,
@@ -163,7 +166,7 @@ pub enum TestEventKind<'a> {
         no_capture: bool,
 
         /// The execution status of the setup script.
-        run_status: SetupScriptExecuteStatus<ChildSingleOutput>,
+        run_status: SetupScriptExecuteStatus<LiveSpec>,
     },
 
     // TODO: add events for BinaryStarted and BinaryFinished? May want a slightly different way to
@@ -216,7 +219,7 @@ pub enum TestEventKind<'a> {
         test_instance: TestInstanceId<'a>,
 
         /// The status of this attempt to run the test. Will never be success.
-        run_status: ExecuteStatus<ChildSingleOutput>,
+        run_status: ExecuteStatus<LiveSpec>,
 
         /// The delay before the next attempt to run the test.
         delay_before_next_attempt: Duration,
@@ -267,7 +270,7 @@ pub enum TestEventKind<'a> {
         junit_store_failure_output: bool,
 
         /// Information about all the runs for this test.
-        run_statuses: ExecutionStatuses<ChildSingleOutput>,
+        run_statuses: ExecutionStatuses<LiveSpec>,
 
         /// Current statistics for number of tests so far.
         current_stats: RunStats,
@@ -398,11 +401,34 @@ pub enum TestEventKind<'a> {
 
         /// Statistics for the run, or overall statistics for stress tests.
         run_stats: RunFinishedStats,
+
+        /// Tests that were expected to run but were not seen during this run.
+        ///
+        /// This is only set for reruns when some tests from the outstanding set
+        /// did not produce any events.
+        outstanding_not_seen: Option<TestsNotSeen>,
     },
 }
 
-/// Progress for a stress test.
+/// Tests that were expected to run but were not seen during a rerun.
 #[derive(Clone, Debug)]
+pub struct TestsNotSeen {
+    /// A sample of test instance IDs that were not seen, up to a reasonable
+    /// limit.
+    ///
+    /// This uses [`OwnedTestInstanceId`] rather than [`TestInstanceId`]
+    /// because the tests may not be present in the current test list (they
+    /// come from the expected outstanding set from a prior run).
+    pub not_seen: Vec<OwnedTestInstanceId>,
+
+    /// The total number of tests not seen (may exceed `not_seen.len()`).
+    pub total_not_seen: usize,
+}
+
+/// Progress for a stress test.
+#[derive(Clone, Copy, Debug, PartialEq, Eq, Serialize, Deserialize)]
+#[serde(tag = "progress-type", rename_all = "kebab-case")]
+#[cfg_attr(test, derive(test_strategy::Arbitrary))]
 pub enum StressProgress {
     /// This is a count-based stress run.
     Count {
@@ -478,7 +504,9 @@ pub enum StressRemaining {
 }
 
 /// The index of the current stress run.
-#[derive(Clone, Copy, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
+#[derive(Clone, Copy, Debug, Eq, Hash, Ord, PartialEq, PartialOrd, Serialize, Deserialize)]
+#[serde(rename_all = "kebab-case")]
+#[cfg_attr(test, derive(test_strategy::Arbitrary))]
 pub struct StressIndex {
     /// The 0-indexed index.
     pub current: u32,
@@ -488,7 +516,9 @@ pub struct StressIndex {
 }
 
 /// Statistics for a completed test run or stress run.
-#[derive(Clone, Debug)]
+#[derive(Clone, Copy, Debug, PartialEq, Eq, Serialize, Deserialize)]
+#[serde(tag = "type", rename_all = "kebab-case")]
+#[cfg_attr(test, derive(test_strategy::Arbitrary))]
 pub enum RunFinishedStats {
     /// A single test run was completed.
     Single(RunStats),
@@ -509,7 +539,9 @@ impl RunFinishedStats {
 }
 
 /// Statistics for a test run.
-#[derive(Copy, Clone, Default, Debug, Eq, PartialEq)]
+#[derive(Copy, Clone, Default, Debug, Eq, PartialEq, Serialize, Deserialize)]
+#[serde(rename_all = "kebab-case")]
+#[cfg_attr(test, derive(test_strategy::Arbitrary))]
 pub struct RunStats {
     /// The total number of tests that were expected to be run at the beginning.
     ///
@@ -646,10 +678,7 @@ impl RunStats {
         }
     }
 
-    pub(crate) fn on_setup_script_finished(
-        &mut self,
-        status: &SetupScriptExecuteStatus<ChildSingleOutput>,
-    ) {
+    pub(crate) fn on_setup_script_finished(&mut self, status: &SetupScriptExecuteStatus<LiveSpec>) {
         self.setup_scripts_finished_count += 1;
 
         match status.result {
@@ -675,7 +704,7 @@ impl RunStats {
         }
     }
 
-    pub(crate) fn on_test_finished(&mut self, run_statuses: &ExecutionStatuses<ChildSingleOutput>) {
+    pub(crate) fn on_test_finished(&mut self, run_statuses: &ExecutionStatuses<LiveSpec>) {
         self.finished_count += 1;
         // run_statuses is guaranteed to have at least one element.
         // * If the last element is success, treat it as success (and possibly flaky).
@@ -743,7 +772,9 @@ impl RunStats {
 }
 
 /// A type summarizing the possible outcomes of a test run.
-#[derive(Copy, Clone, Debug, Eq, PartialEq)]
+#[derive(Copy, Clone, Debug, Eq, PartialEq, Serialize, Deserialize)]
+#[serde(tag = "outcome", rename_all = "kebab-case")]
+#[cfg_attr(test, derive(test_strategy::Arbitrary))]
 pub enum FinalRunStats {
     /// The test run was successful, or is successful so far.
     Success,
@@ -771,7 +802,9 @@ pub enum FinalRunStats {
 }
 
 /// Statistics for a stress run.
-#[derive(Clone, Debug)]
+#[derive(Clone, Copy, Debug, PartialEq, Eq, Serialize, Deserialize)]
+#[serde(rename_all = "kebab-case")]
+#[cfg_attr(test, derive(test_strategy::Arbitrary))]
 pub struct StressRunStats {
     /// The number of stress runs completed.
     pub completed: StressIndex,
@@ -817,7 +850,9 @@ pub enum StressFinalRunStats {
 }
 
 /// A type summarizing the step at which a test run failed.
-#[derive(Copy, Clone, Debug, Eq, PartialEq)]
+#[derive(Copy, Clone, Debug, Eq, PartialEq, Serialize, Deserialize)]
+#[serde(tag = "step", rename_all = "kebab-case")]
+#[cfg_attr(test, derive(test_strategy::Arbitrary))]
 pub enum RunStatsFailureKind {
     /// The run was interrupted during setup script execution.
     SetupScript,
@@ -835,30 +870,66 @@ pub enum RunStatsFailureKind {
 
 /// Information about executions of a test, including retries.
 ///
-/// The type parameter `O` represents how test output is stored.
-#[derive(Clone, Debug)]
-pub struct ExecutionStatuses<O> {
+/// The type parameter `S` specifies how test output is stored (see
+/// [`OutputSpec`]).
+#[derive_where::derive_where(Clone, Debug, PartialEq, Eq; S::ChildOutputDesc)]
+#[derive(Serialize)]
+#[serde(
+    rename_all = "kebab-case",
+    bound(serialize = "S: SerializableOutputSpec")
+)]
+#[cfg_attr(
+    test,
+    derive(test_strategy::Arbitrary),
+    arbitrary(bound(S: ArbitraryOutputSpec))
+)]
+pub struct ExecutionStatuses<S: OutputSpec> {
     /// This is guaranteed to be non-empty.
-    statuses: Vec<ExecuteStatus<O>>,
+    #[cfg_attr(test, strategy(proptest::collection::vec(proptest::arbitrary::any::<ExecuteStatus<S>>(), 1..=3)))]
+    statuses: Vec<ExecuteStatus<S>>,
+}
+
+impl<'de, S: SerializableOutputSpec> Deserialize<'de> for ExecutionStatuses<S> {
+    fn deserialize<D: serde::Deserializer<'de>>(deserializer: D) -> Result<Self, D::Error> {
+        // Deserialize as the wrapper struct that matches the Serialize output.
+        // S is already bound as SerializableOutputSpec on this impl.
+        #[derive(Deserialize)]
+        #[serde(
+            rename_all = "kebab-case",
+            bound(deserialize = "S: SerializableOutputSpec")
+        )]
+        struct Helper<S: OutputSpec> {
+            statuses: Vec<ExecuteStatus<S>>,
+        }
+
+        let helper = Helper::<S>::deserialize(deserializer)?;
+        if helper.statuses.is_empty() {
+            return Err(serde::de::Error::custom("expected non-empty statuses"));
+        }
+        Ok(Self {
+            statuses: helper.statuses,
+        })
+    }
 }
 
 #[expect(clippy::len_without_is_empty)] // RunStatuses is never empty
-impl<O> ExecutionStatuses<O> {
-    pub(crate) fn new(statuses: Vec<ExecuteStatus<O>>) -> Self {
+impl<S: OutputSpec> ExecutionStatuses<S> {
+    pub(crate) fn new(statuses: Vec<ExecuteStatus<S>>) -> Self {
+        debug_assert!(!statuses.is_empty(), "ExecutionStatuses must be non-empty");
         Self { statuses }
     }
 
     /// Returns the last execution status.
     ///
     /// This status is typically used as the final result.
-    pub fn last_status(&self) -> &ExecuteStatus<O> {
+    pub fn last_status(&self) -> &ExecuteStatus<S> {
         self.statuses
             .last()
             .expect("execution statuses is non-empty")
     }
 
     /// Iterates over all the statuses.
-    pub fn iter(&self) -> impl DoubleEndedIterator<Item = &'_ ExecuteStatus<O>> + '_ {
+    pub fn iter(&self) -> impl DoubleEndedIterator<Item = &'_ ExecuteStatus<S>> + '_ {
         self.statuses.iter()
     }
 
@@ -868,7 +939,7 @@ impl<O> ExecutionStatuses<O> {
     }
 
     /// Returns a description of self.
-    pub fn describe(&self) -> ExecutionDescription<'_, O> {
+    pub fn describe(&self) -> ExecutionDescription<'_, S> {
         let last_status = self.last_status();
         if last_status.result.is_success() {
             if self.statuses.len() > 1 {
@@ -896,54 +967,64 @@ impl<O> ExecutionStatuses<O> {
     }
 }
 
+impl<S: OutputSpec> IntoIterator for ExecutionStatuses<S> {
+    type Item = ExecuteStatus<S>;
+    type IntoIter = std::vec::IntoIter<ExecuteStatus<S>>;
+
+    fn into_iter(self) -> Self::IntoIter {
+        self.statuses.into_iter()
+    }
+}
+
 /// A description of test executions obtained from `ExecuteStatuses`.
 ///
 /// This can be used to quickly determine whether a test passed, failed or was flaky.
 ///
-/// The type parameter `O` represents how test output is stored.
-#[derive(Debug)]
-pub enum ExecutionDescription<'a, O> {
+/// The type parameter `S` specifies how test output is stored (see
+/// [`OutputSpec`]).
+#[derive_where::derive_where(Debug; S::ChildOutputDesc)]
+pub enum ExecutionDescription<'a, S: OutputSpec> {
     /// The test was run once and was successful.
     Success {
         /// The status of the test.
-        single_status: &'a ExecuteStatus<O>,
+        single_status: &'a ExecuteStatus<S>,
     },
 
     /// The test was run more than once. The final result was successful.
     Flaky {
         /// The last, successful status.
-        last_status: &'a ExecuteStatus<O>,
+        last_status: &'a ExecuteStatus<S>,
 
         /// Previous statuses, none of which are successes.
-        prior_statuses: &'a [ExecuteStatus<O>],
+        prior_statuses: &'a [ExecuteStatus<S>],
     },
 
     /// The test was run once, or possibly multiple times. All runs failed.
     Failure {
         /// The first, failing status.
-        first_status: &'a ExecuteStatus<O>,
+        first_status: &'a ExecuteStatus<S>,
 
         /// The last, failing status. Same as the first status if no retries were performed.
-        last_status: &'a ExecuteStatus<O>,
+        last_status: &'a ExecuteStatus<S>,
 
         /// Any retries that were performed. All of these runs failed.
         ///
         /// May be empty.
-        retries: &'a [ExecuteStatus<O>],
+        retries: &'a [ExecuteStatus<S>],
     },
 }
 
-// Manual Copy and Clone implementations to avoid requiring O: Copy/Clone, since
-// ExecutionDescription just stores references.
-impl<O> Clone for ExecutionDescription<'_, O> {
+// Manual Copy and Clone implementations to avoid requiring S::ChildOutputDesc:
+// Copy/Clone, since ExecutionDescription only stores references.
+impl<S: OutputSpec> Clone for ExecutionDescription<'_, S> {
     fn clone(&self) -> Self {
         *self
     }
 }
 
-impl<O> Copy for ExecutionDescription<'_, O> {}
+impl<S: OutputSpec> Copy for ExecutionDescription<'_, S> {}
 
-impl<'a, O> ExecutionDescription<'a, O> {
+impl<'a, S: OutputSpec> ExecutionDescription<'a, S> {
     /// Returns the status level for this `ExecutionDescription`.
     pub fn status_level(&self) -> StatusLevel {
         match self {
@@ -997,7 +1078,7 @@ impl<'a, O> ExecutionDescription<'a, O> {
     }
 
     /// Returns the last run status.
-    pub fn last_status(&self) -> &'a ExecuteStatus<O> {
+    pub fn last_status(&self) -> &'a ExecuteStatus<S> {
         match self {
             ExecutionDescription::Success {
                 single_status: last_status,
@@ -1013,7 +1094,9 @@ impl<'a, O> ExecutionDescription<'a, O> {
 /// This contains the formatted error messages, pre-computed from the execution
 /// output and result. Useful for record-replay scenarios where the rendering
 /// is done on the server.
-#[derive(Clone, Debug)]
+#[derive(Clone, Debug, PartialEq, Eq, Serialize, Deserialize)]
+#[serde(rename_all = "kebab-case")]
+#[cfg_attr(test, derive(test_strategy::Arbitrary))]
 pub struct ErrorSummary {
     /// A short summary of the error, suitable for display in a single line.
     pub short_message: String,
@@ -1026,7 +1109,9 @@ pub struct ErrorSummary {
 ///
 /// This contains an error message heuristically extracted from test output,
 /// such as a panic message or error string.
-#[derive(Clone, Debug)]
+#[derive(Clone, Debug, PartialEq, Eq, Serialize, Deserialize)]
+#[serde(rename_all = "kebab-case")]
+#[cfg_attr(test, derive(test_strategy::Arbitrary))]
 pub struct OutputErrorSlice {
     /// The extracted error slice as a string.
     pub slice: String,
@@ -1041,24 +1126,42 @@ pub struct OutputErrorSlice {
 /// [`ExecutionResultDescription`], a platform-independent type that can be
 /// serialized and deserialized across platforms.
 ///
-/// The type parameter `O` represents how test output is stored:
-/// - [`ChildSingleOutput`]: Output stored in memory with lazy string conversion.
-/// - Other types may be used for serialization to archives.
-#[derive(Clone, Debug)]
-pub struct ExecuteStatus<O> {
+/// The type parameter `S` specifies how test output is stored (see
+/// [`OutputSpec`]).
+#[derive_where::derive_where(Clone, Debug, PartialEq, Eq; S::ChildOutputDesc)]
+#[derive(Serialize, Deserialize)]
+#[serde(
+    rename_all = "kebab-case",
+    bound(
+        serialize = "S: SerializableOutputSpec",
+        deserialize = "S: SerializableOutputSpec"
+    )
+)]
+#[cfg_attr(
+    test,
+    derive(test_strategy::Arbitrary),
+    arbitrary(bound(S: ArbitraryOutputSpec))
+)]
+pub struct ExecuteStatus<S: OutputSpec> {
     /// Retry-related data.
     pub retry_data: RetryData,
     /// The stdout and stderr output for this test.
-    pub output: ChildExecutionOutputDescription<O>,
+    pub output: ChildExecutionOutputDescription<S>,
     /// The execution result for this test: pass, fail or execution error.
     pub result: ExecutionResultDescription,
     /// The time at which the test started.
+    #[cfg_attr(
+        test,
+        strategy(crate::reporter::test_helpers::arb_datetime_fixed_offset())
+    )]
     pub start_time: DateTime<FixedOffset>,
     /// The time it took for the test to run.
+    #[cfg_attr(test, strategy(crate::reporter::test_helpers::arb_duration()))]
     pub time_taken: Duration,
     /// Whether this test counts as slow.
     pub is_slow: bool,
     /// The delay will be non-zero if this is a retry and delay was specified.
+    #[cfg_attr(test, strategy(crate::reporter::test_helpers::arb_duration()))]
     pub delay_before_start: Duration,
     /// Pre-computed error summary, if available.
     ///
@@ -1078,19 +1181,38 @@ pub struct ExecuteStatus<O> {
 /// [`ExecutionResultDescription`], a platform-independent type that can be
 /// serialized and deserialized across platforms.
 ///
-/// The type parameter `O` represents how test output is stored.
-#[derive(Clone, Debug)]
-pub struct SetupScriptExecuteStatus<O> {
+/// The type parameter `S` specifies how test output is stored (see
+/// [`OutputSpec`]).
+#[derive_where::derive_where(Clone, Debug, PartialEq, Eq; S::ChildOutputDesc)]
+#[derive(Serialize, Deserialize)]
+#[serde(
+    rename_all = "kebab-case",
+    bound(
+        serialize = "S: SerializableOutputSpec",
+        deserialize = "S: SerializableOutputSpec"
+    )
+)]
+#[cfg_attr(
+    test,
+    derive(test_strategy::Arbitrary),
+    arbitrary(bound(S: ArbitraryOutputSpec))
+)]
+pub struct SetupScriptExecuteStatus<S: OutputSpec> {
     /// Output for this setup script.
-    pub output: ChildExecutionOutputDescription<O>,
+    pub output: ChildExecutionOutputDescription<S>,
 
     /// The execution result for this setup script: pass, fail or execution error.
     pub result: ExecutionResultDescription,
 
     /// The time at which the script started.
+    #[cfg_attr(
+        test,
+        strategy(crate::reporter::test_helpers::arb_datetime_fixed_offset())
+    )]
     pub start_time: DateTime<FixedOffset>,
 
     /// The time it took for the script to run.
+    #[cfg_attr(test, strategy(crate::reporter::test_helpers::arb_duration()))]
     pub time_taken: Duration,
 
     /// Whether this script counts as slow.
@@ -1112,7 +1234,9 @@ pub struct SetupScriptExecuteStatus<O> {
 /// A map of environment variables set by a setup script.
 ///
 /// Part of [`SetupScriptExecuteStatus`].
-#[derive(Clone, Debug)]
+#[derive(Clone, Debug, PartialEq, Eq, Serialize, Deserialize)]
+#[serde(rename_all = "kebab-case")]
+#[cfg_attr(test, derive(test_strategy::Arbitrary))]
 pub struct SetupScriptEnvMap {
     /// The map of environment variables set by the script.
     pub env_map: BTreeMap<String, String>,
@@ -1125,12 +1249,23 @@ pub struct SetupScriptEnvMap {
 /// The result of executing a child process, generic over output storage.
 ///
 /// This is the external-facing counterpart to [`ChildExecutionOutput`]. The
-/// type parameter `O` represents how output is stored:
-///
-/// - [`ChildSingleOutput`]: Output stored in memory with lazy string caching.
-///   Used by reporter event types during live runs.
-#[derive(Clone, Debug)]
-pub enum ChildExecutionOutputDescription<O> {
+/// type parameter `S` specifies how output is stored (see [`OutputSpec`]).
+#[derive_where::derive_where(Clone, Debug, PartialEq, Eq; S::ChildOutputDesc)]
+#[derive(Serialize, Deserialize)]
+#[serde(
+    tag = "type",
+    rename_all = "kebab-case",
+    bound(
+        serialize = "S: SerializableOutputSpec",
+        deserialize = "S: SerializableOutputSpec"
+    )
+)]
+#[cfg_attr(
+    test,
+    derive(test_strategy::Arbitrary),
+    arbitrary(bound(S: ArbitraryOutputSpec))
+)]
+pub enum ChildExecutionOutputDescription<S: OutputSpec> {
     /// The process was run and the output was captured.
     Output {
         /// If the process has finished executing, the final state it is in.
@@ -1139,7 +1274,7 @@ pub enum ChildExecutionOutputDescription<O> {
         result: Option<ExecutionResultDescription>,
 
         /// The captured output.
-        output: ChildOutputDescription<O>,
+        output: S::ChildOutputDesc,
 
         /// Errors that occurred while waiting on the child process or parsing
         /// its output.
@@ -1150,7 +1285,7 @@ pub enum ChildExecutionOutputDescription<O> {
     StartError(ChildStartErrorDescription),
 }
 
-impl<O> ChildExecutionOutputDescription<O> {
+impl<S: OutputSpec> ChildExecutionOutputDescription<S> {
     /// Returns true if there are any errors in this output.
     pub fn has_errors(&self) -> bool {
         match self {
@@ -1168,30 +1303,42 @@ impl<O> ChildExecutionOutputDescription<O> {
     }
 }
 
-/// The output of a child process, generic over output storage.
+/// The output of a child process during live execution.
 ///
 /// This represents either split stdout/stderr or combined output. The `Option`
 /// wrappers distinguish between "not captured" (`None`) and "captured but
 /// empty" (`Some` with empty content).
-#[derive(Clone, Debug, Serialize, Deserialize)]
-#[serde(tag = "kind", rename_all = "kebab-case")]
-pub enum ChildOutputDescription<O> {
+///
+/// The `NotLoaded` variant is used during replay when the display
+/// configuration indicates that output won't be shown.
+///
+/// For the recording counterpart, see
+/// [`ZipStoreOutputDescription`](crate::record::ZipStoreOutputDescription).
+#[derive(Clone, Debug)]
+pub enum ChildOutputDescription {
     /// The output was split into stdout and stderr.
     Split {
         /// Standard output, or `None` if not captured.
-        stdout: Option<O>,
+        stdout: Option<ChildSingleOutput>,
         /// Standard error, or `None` if not captured.
-        stderr: Option<O>,
+        stderr: Option<ChildSingleOutput>,
     },
 
     /// The output was combined into a single stream.
     Combined {
         /// The combined output.
-        output: O,
+        output: ChildSingleOutput,
     },
+
+    /// Output exists but was not loaded.
+    ///
+    /// This variant is used during replay when the display configuration
+    /// indicates that output won't be shown. Code that accesses output
+    /// bytes must never be reached with this variant.
+    NotLoaded,
 }
 
-impl ChildOutputDescription<ChildSingleOutput> {
+impl ChildOutputDescription {
     /// Returns the lengths of stdout and stderr in bytes.
     ///
     /// Returns `None` for each stream that wasn't captured.
@@ -1202,6 +1349,13 @@ impl ChildOutputDescription<ChildSingleOutput> {
                 stderr.as_ref().map(|s| s.buf.len() as u64),
             ),
             Self::Combined { output } => (Some(output.buf.len() as u64), None),
+            Self::NotLoaded => {
+                unreachable!(
+                    "attempted to get output lengths from output that was not loaded \
+                     (this method is only called from the live runner, where NotLoaded \
+                     is never produced)"
+                );
+            }
         }
     }
 }
@@ -1209,7 +1363,9 @@ impl ChildOutputDescription<ChildSingleOutput> {
 /// A serializable description of an error that occurred while starting a child process.
 ///
 /// This is the external-facing counterpart to [`ChildStartError`].
-#[derive(Clone, Debug)]
+#[derive(Clone, Debug, PartialEq, Eq, Serialize, Deserialize)]
+#[serde(tag = "kind", rename_all = "kebab-case")]
+#[cfg_attr(test, derive(test_strategy::Arbitrary))]
 pub enum ChildStartErrorDescription {
     /// An error occurred while creating a temporary path for a setup script.
     TempPath {
@@ -1246,7 +1402,9 @@ impl std::error::Error for ChildStartErrorDescription {
 /// A serializable description of an error that occurred while managing a child process.
 ///
 /// This is the external-facing counterpart to [`ChildError`].
-#[derive(Clone, Debug)]
+#[derive(Clone, Debug, PartialEq, Eq, Serialize, Deserialize)]
+#[serde(tag = "kind", rename_all = "kebab-case")]
+#[cfg_attr(test, derive(test_strategy::Arbitrary))]
 pub enum ChildErrorDescription {
     /// An error occurred while reading standard output.
     ReadStdout {
@@ -1312,7 +1470,8 @@ impl std::error::Error for ChildErrorDescription {
 /// A serializable description of an I/O error.
 ///
 /// This captures the error message from an [`std::io::Error`].
-#[derive(Clone, Debug)]
+#[derive(Clone, Debug, PartialEq, Eq, Serialize, Deserialize)]
+#[cfg_attr(test, derive(test_strategy::Arbitrary))]
 pub struct IoErrorDescription {
     message: String,
 }
@@ -1325,7 +1484,7 @@ impl fmt::Display for IoErrorDescription {
 
 impl std::error::Error for IoErrorDescription {}
 
-impl From<ChildExecutionOutput> for ChildExecutionOutputDescription<ChildSingleOutput> {
+impl From<ChildExecutionOutput> for ChildExecutionOutputDescription<LiveSpec> {
     fn from(output: ChildExecutionOutput) -> Self {
         match output {
             ChildExecutionOutput::Output {
@@ -1344,7 +1503,7 @@ impl From<ChildExecutionOutput> for ChildExecutionOutputDescription<ChildSingleO
     }
 }
 
-impl From<ChildOutput> for ChildOutputDescription<ChildSingleOutput> {
+impl From<ChildOutput> for ChildOutputDescription {
     fn from(output: ChildOutput) -> Self {
         match output {
             ChildOutput::Split(split) => Self::Split {
@@ -1406,7 +1565,9 @@ impl From<ChildError> for ChildErrorDescription {
 }
 
 /// Data related to retries for a test.
-#[derive(Clone, Copy, Debug, Eq, PartialEq, PartialOrd, Ord)]
+#[derive(Clone, Copy, Debug, Eq, PartialEq, PartialOrd, Ord, Serialize, Deserialize)]
+#[serde(rename_all = "kebab-case")]
+#[cfg_attr(test, derive(test_strategy::Arbitrary))]
 pub struct RetryData {
     /// The current attempt. In the range `[1, total_attempts]`.
     pub attempt: u32,
@@ -1574,6 +1735,7 @@ impl fmt::Debug for AbortStatus {
 /// platform-specific lookups.
 #[derive(Clone, Debug, Eq, PartialEq, Serialize, Deserialize)]
 #[serde(tag = "kind", rename_all = "kebab-case")]
+#[cfg_attr(test, derive(test_strategy::Arbitrary))]
 #[non_exhaustive]
 pub enum AbortDescription {
     /// The process was aborted by a Unix signal.
@@ -1582,6 +1744,10 @@ pub enum AbortDescription {
         signal: i32,
         /// The signal name without the "SIG" prefix (e.g., "TERM", "SEGV"),
         /// if known.
+        #[cfg_attr(
+            test,
+            strategy(proptest::option::of(crate::reporter::test_helpers::arb_smol_str()))
+        )]
         name: Option<SmolStr>,
     },
 
@@ -1590,6 +1756,10 @@ pub enum AbortDescription {
         /// The NTSTATUS code.
         code: i32,
         /// The human-readable message from the Win32 error code, if available.
+        #[cfg_attr(
+            test,
+            strategy(proptest::option::of(crate::reporter::test_helpers::arb_smol_str()))
+        )]
         message: Option<SmolStr>,
     },
 
@@ -1651,6 +1821,7 @@ impl From<AbortStatus> for AbortDescription {
 /// This is the platform-independent counterpart to [`FailureStatus`].
 #[derive(Clone, Debug, Eq, PartialEq, Serialize, Deserialize)]
 #[serde(tag = "kind", rename_all = "kebab-case")]
+#[cfg_attr(test, derive(test_strategy::Arbitrary))]
 #[non_exhaustive]
 pub enum FailureDescription {
     /// The test exited with a non-zero exit code.
@@ -1697,6 +1868,7 @@ impl fmt::Display for FailureDescription {
 /// in external-facing types like [`ExecuteStatus`].
 #[derive(Clone, Debug, Eq, PartialEq, Serialize, Deserialize)]
 #[serde(tag = "status", rename_all = "kebab-case")]
+#[cfg_attr(test, derive(test_strategy::Arbitrary))]
 #[non_exhaustive]
 pub enum ExecutionResultDescription {
     /// The test passed.
@@ -1812,7 +1984,8 @@ impl From<ExecutionResult> for ExecutionResultDescription {
 
 // Note: the order here matters -- it indicates severity of cancellation
 /// The reason why a test run is being cancelled.
-#[derive(Copy, Clone, Debug, Eq, PartialEq, Ord, PartialOrd)]
+#[derive(Copy, Clone, Debug, Eq, PartialEq, Ord, PartialOrd, Deserialize, Serialize)]
+#[serde(rename_all = "kebab-case")]
 #[cfg_attr(test, derive(test_strategy::Arbitrary))]
 pub enum CancelReason {
     /// A setup script failed.
@@ -1890,7 +2063,7 @@ impl UnitKind {
 #[derive(Clone, Debug)]
 pub enum InfoResponse<'a> {
     /// A setup script's response.
-    SetupScript(SetupScriptInfoResponse<'a>),
+    SetupScript(SetupScriptInfoResponse),
 
     /// A test's response.
     Test(TestInfoResponse<'a>),
@@ -1898,7 +2071,7 @@ pub enum InfoResponse<'a> {
 
 /// A setup script's response to an information request.
 #[derive(Clone, Debug)]
-pub struct SetupScriptInfoResponse<'a> {
+pub struct SetupScriptInfoResponse {
     /// The stress index of the setup script.
     pub stress_index: Option<StressIndex>,
 
@@ -1909,13 +2082,13 @@ pub struct SetupScriptInfoResponse<'a> {
     pub program: String,
 
     /// The list of arguments to the program.
-    pub args: &'a [String],
+    pub args: Vec<String>,
 
     /// The state of the setup script.
     pub state: UnitState,
 
     /// Output obtained from the setup script.
-    pub output: ChildExecutionOutputDescription<ChildSingleOutput>,
+    pub output: ChildExecutionOutputDescription<LiveSpec>,
 }
 
 /// A test's response to an information request.
@@ -1934,7 +2107,7 @@ pub struct TestInfoResponse<'a> {
     pub state: UnitState,
 
     /// Output obtained from the test.
-    pub output: ChildExecutionOutputDescription<ChildSingleOutput>,
+    pub output: ChildExecutionOutputDescription<LiveSpec>,
 }
 
 /// The current state of a test or script process: running, exiting, or
