@@ -26,12 +26,16 @@ use crate::{
     config::elements::{LeakTimeoutResult, SlowTimeoutResult},
     errors::{DisplayErrorChain, FormatVersionError, FormatVersionErrorInner, WriteEventError},
     list::{RustTestSuite, TestList},
-    reporter::events::{ExecutionResult, StressIndex, TestEvent, TestEventKind},
-    test_output::{ChildExecutionOutput, ChildOutput, ChildSingleOutput},
+    output_spec::LiveSpec,
+    reporter::events::{
+        ChildExecutionOutputDescription, ChildOutputDescription, ExecutionResultDescription,
+        StressIndex, TestEvent, TestEventKind,
+    },
+    test_output::ChildSingleOutput,
 };
 use bstr::ByteSlice;
 use iddqd::{IdOrdItem, IdOrdMap, id_ord_map, id_upcast};
-use nextest_metadata::{MismatchReason, RustBinaryId};
+use nextest_metadata::{MismatchReason, RustBinaryId, TestCaseName};
 use std::fmt::Write as _;
 
 /// To support pinning the version of the output, we just use this simple enum
@@ -269,20 +273,20 @@ impl<'cfg> LibtestReporter<'cfg> {
 
                 (
                     KIND_TEST,
-                    match run_statuses.last_status().result {
-                        ExecutionResult::Pass
-                        | ExecutionResult::Timeout {
+                    match &run_statuses.last_status().result {
+                        ExecutionResultDescription::Pass
+                        | ExecutionResultDescription::Timeout {
                             result: SlowTimeoutResult::Pass,
                         }
-                        | ExecutionResult::Leak {
+                        | ExecutionResultDescription::Leak {
                             result: LeakTimeoutResult::Pass,
                         } => EVENT_OK,
-                        ExecutionResult::Leak {
+                        ExecutionResultDescription::Leak {
                             result: LeakTimeoutResult::Fail,
                         }
-                        | ExecutionResult::Fail { .. }
-                        | ExecutionResult::ExecFail
-                        | ExecutionResult::Timeout {
+                        | ExecutionResultDescription::Fail { .. }
+                        | ExecutionResultDescription::ExecFail
+                        | ExecutionResultDescription::Timeout {
                             result: SlowTimeoutResult::Fail,
                         } => EVENT_FAILED,
                     },
@@ -428,8 +432,9 @@ impl<'cfg> LibtestReporter<'cfg> {
                 )
                 .map_err(fmt_err)?;
 
-                match last_status.result {
-                    ExecutionResult::Fail { .. } | ExecutionResult::ExecFail => {
+                match &last_status.result {
+                    ExecutionResultDescription::Fail { .. }
+                    | ExecutionResultDescription::ExecFail => {
                         test_suite_mut.failed += 1;
 
                         // Write the output from the test into the `stdout` (even
@@ -443,7 +448,7 @@ impl<'cfg> LibtestReporter<'cfg> {
                         )?;
                         out.extend_from_slice(b"\"");
                     }
-                    ExecutionResult::Timeout {
+                    ExecutionResultDescription::Timeout {
                         result: SlowTimeoutResult::Fail,
                     } => {
                         test_suite_mut.failed += 1;
@@ -582,21 +587,21 @@ impl<'cfg> LibtestReporter<'cfg> {
 /// This function relies on the fact that nextest runs every individual test in
 /// isolation.
 fn strip_human_output_from_failed_test(
-    output: &ChildExecutionOutput,
+    output: &ChildExecutionOutputDescription<LiveSpec>,
     out: &mut bytes::BytesMut,
-    test_name: &str,
+    test_name: &TestCaseName,
 ) -> Result<(), WriteEventError> {
     match output {
-        ChildExecutionOutput::Output {
+        ChildExecutionOutputDescription::Output {
             result: _,
             output,
             errors,
         } => {
             match output {
-                ChildOutput::Combined { output } => {
+                ChildOutputDescription::Combined { output } => {
                     strip_human_stdout_or_combined(output, out, test_name)?;
                 }
-                ChildOutput::Split(split) => {
+                ChildOutputDescription::Split { stdout, stderr } => {
                     // This is not a case that we hit because we always set CaptureStrategy to Combined. But
                     // handle it in a reasonable fashion. (We do have a unit test for this case, so gate the
                     // assertion with cfg(not(test)).)
@@ -604,7 +609,7 @@ fn strip_human_output_from_failed_test(
                     {
                         debug_assert!(false, "libtest output requires CaptureStrategy::Combined");
                     }
-                    if let Some(stdout) = &split.stdout {
+                    if let Some(stdout) = stdout {
                         if !stdout.is_empty() {
                             write!(out, "--- STDOUT ---\\n").map_err(fmt_err)?;
                             strip_human_stdout_or_combined(stdout, out, test_name)?;
@@ -613,7 +618,7 @@ fn strip_human_output_from_failed_test(
                         write!(out, "(stdout not captured)").map_err(fmt_err)?;
                     }
                     // If stderr is not empty, just write all of it in.
-                    if let Some(stderr) = &split.stderr {
+                    if let Some(stderr) = stderr {
                         if !stderr.is_empty() {
                             write!(out, "\\n--- STDERR ---\\n").map_err(fmt_err)?;
                             write!(out, "{}", EscapedString(stderr.as_str_lossy()))
@@ -623,6 +628,13 @@ fn strip_human_output_from_failed_test(
                         writeln!(out, "\\n(stderr not captured)").map_err(fmt_err)?;
                     }
                 }
+                ChildOutputDescription::NotLoaded => {
+                    unreachable!(
+                        "attempted to strip output from output that was not loaded \
+                         (the libtest reporter is not used during replay, where NotLoaded \
+                         is produced)"
+                    );
+                }
             }
 
             if let Some(errors) = errors {
@@ -635,7 +647,7 @@ fn strip_human_output_from_failed_test(
                 .map_err(fmt_err)?;
             }
         }
-        ChildExecutionOutput::StartError(error) => {
+        ChildExecutionOutputDescription::StartError(error) => {
             write!(out, "--- EXECUTION ERROR ---\\n").map_err(fmt_err)?;
             write!(
                 out,
@@ -651,7 +663,7 @@ fn strip_human_output_from_failed_test(
 fn strip_human_stdout_or_combined(
     output: &ChildSingleOutput,
     out: &mut bytes::BytesMut,
-    test_name: &str,
+    test_name: &TestCaseName,
 ) -> Result<(), WriteEventError> {
     if output.buf.contains_str("running 1 test\n") {
         // This is most likely the default test harness.
@@ -758,11 +770,15 @@ impl std::fmt::Display for EscapedString<'_> {
 mod test {
     use crate::{
         errors::ChildStartError,
-        reporter::structured::libtest::strip_human_output_from_failed_test,
+        reporter::{
+            events::ChildExecutionOutputDescription,
+            structured::libtest::strip_human_output_from_failed_test,
+        },
         test_output::{ChildExecutionOutput, ChildOutput, ChildSplitOutput},
     };
     use bytes::BytesMut;
     use color_eyre::eyre::eyre;
+    use nextest_metadata::TestCaseName;
     use std::{io, sync::Arc};
 
     /// Validates that the human output portion from a failed test is stripped
@@ -810,14 +826,16 @@ note: Some details are omitted, run with `RUST_BACKTRACE=full` for a verbose bac
         };
 
         let mut actual = bytes::BytesMut::new();
+        let output_desc: ChildExecutionOutputDescription<_> = ChildExecutionOutput::Output {
+            result: None,
+            output,
+            errors: None,
+        }
+        .into();
         strip_human_output_from_failed_test(
-            &ChildExecutionOutput::Output {
-                result: None,
-                output,
-                errors: None,
-            },
+            &output_desc,
             &mut actual,
-            "index::test::download_url_crates_io",
+            &TestCaseName::new("index::test::download_url_crates_io"),
         )
         .unwrap();
 
@@ -841,14 +859,16 @@ note: Some details are omitted, run with `RUST_BACKTRACE=full` for a verbose bac
         };
 
         let mut actual = bytes::BytesMut::new();
+        let output_desc: ChildExecutionOutputDescription<_> = ChildExecutionOutput::Output {
+            result: None,
+            output,
+            errors: None,
+        }
+        .into();
         strip_human_output_from_failed_test(
-            &ChildExecutionOutput::Output {
-                result: None,
-                output,
-                errors: None,
-            },
+            &output_desc,
             &mut actual,
-            "non-existent",
+            &TestCaseName::new("non-existent"),
         )
         .unwrap();
 
@@ -860,10 +880,16 @@ note: Some details are omitted, run with `RUST_BACKTRACE=full` for a verbose bac
         let inner_error = eyre!("inner error");
         let error = io::Error::other(inner_error);
 
-        let output = ChildExecutionOutput::StartError(ChildStartError::Spawn(Arc::new(error)));
+        let output: ChildExecutionOutputDescription<_> =
+            ChildExecutionOutput::StartError(ChildStartError::Spawn(Arc::new(error))).into();
 
         let mut actual = bytes::BytesMut::new();
-        strip_human_output_from_failed_test(&output, &mut actual, "non-existent").unwrap();
+        strip_human_output_from_failed_test(
+            &output,
+            &mut actual,
+            &TestCaseName::new("non-existent"),
+        )
+        .unwrap();
 
         insta::assert_snapshot!(std::str::from_utf8(&actual).unwrap());
     }
@@ -871,17 +897,19 @@ note: Some details are omitted, run with `RUST_BACKTRACE=full` for a verbose bac
     #[test]
     fn strips_human_output_none() {
         let mut actual = bytes::BytesMut::new();
+        let output_desc: ChildExecutionOutputDescription<_> = ChildExecutionOutput::Output {
+            result: None,
+            output: ChildOutput::Split(ChildSplitOutput {
+                stdout: None,
+                stderr: None,
+            }),
+            errors: None,
+        }
+        .into();
         strip_human_output_from_failed_test(
-            &ChildExecutionOutput::Output {
-                result: None,
-                output: ChildOutput::Split(ChildSplitOutput {
-                    stdout: None,
-                    stderr: None,
-                }),
-                errors: None,
-            },
+            &output_desc,
             &mut actual,
-            "non-existent",
+            &TestCaseName::new("non-existent"),
         )
         .unwrap();
 
