@@ -15,6 +15,7 @@ use crate::{
     },
     input::{InputEvent, InputHandler},
     list::{OwnedTestInstanceId, TestInstance, TestInstanceId, TestInstanceIdKey, TestList},
+    output_spec::LiveSpec,
     reporter::events::{
         CancelReason, ChildExecutionOutputDescription, ExecuteStatus, ExecutionResultDescription,
         ExecutionStatuses, FailureDescription, FinalRunStats, InfoResponse, ReporterEvent,
@@ -26,7 +27,6 @@ use crate::{
         JobControlEvent, ShutdownEvent, ShutdownSignalEvent, SignalEvent, SignalHandler,
         SignalInfoEvent,
     },
-    test_output::ChildSingleOutput,
     time::StopwatchStart,
 };
 use chrono::Local;
@@ -267,16 +267,6 @@ where
 
                     self.broadcast_request(RunUnitRequest::Signal(SignalRequest::Continue));
                 }
-                #[cfg(not(unix))]
-                HandleEventResponse::JobControl(e) => {
-                    // On platforms other than Unix this enum is expected to be
-                    // empty; we can check this assumption at compile time like
-                    // so.
-                    //
-                    // Rust 1.82 handles empty enums better, and this won't be
-                    // required after we bump the MSRV to that.
-                    match e {}
-                }
                 HandleEventResponse::Info(_) => {
                     // In reality, this is bounded by the number of
                     // tests running at the same time.
@@ -924,8 +914,8 @@ where
     fn finish_test(
         &mut self,
         key: TestInstanceId<'a>,
-        last_run_status: ExecuteStatus<ChildSingleOutput>,
-    ) -> ExecutionStatuses<ChildSingleOutput> {
+        last_run_status: ExecuteStatus<LiveSpec>,
+    ) -> ExecutionStatuses<LiveSpec> {
         self.running_tests
             .remove(&key)
             .unwrap_or_else(|| {
@@ -1397,19 +1387,16 @@ struct ContextTestInstance<'a> {
     // Store the instance primarily for debugging.
     #[expect(dead_code)]
     instance: TestInstance<'a>,
-    past_attempts: Vec<ExecuteStatus<ChildSingleOutput>>,
+    past_attempts: Vec<ExecuteStatus<LiveSpec>>,
     req_tx: UnboundedSender<RunUnitRequest<'a>>,
 }
 
 impl ContextTestInstance<'_> {
-    fn attempt_failed_will_retry(&mut self, run_status: ExecuteStatus<ChildSingleOutput>) {
+    fn attempt_failed_will_retry(&mut self, run_status: ExecuteStatus<LiveSpec>) {
         self.past_attempts.push(run_status);
     }
 
-    fn finish(
-        self,
-        last_run_status: ExecuteStatus<ChildSingleOutput>,
-    ) -> ExecutionStatuses<ChildSingleOutput> {
+    fn finish(self, last_run_status: ExecuteStatus<LiveSpec>) -> ExecutionStatuses<LiveSpec> {
         let mut attempts = self.past_attempts;
         attempts.push(last_run_status);
         ExecutionStatuses::new(attempts)
