@@ -194,7 +194,7 @@ impl<'a> ExecutorContext<'a> {
         resp_tx: UnboundedSender<ExecutorEvent<'a>>,
         setup_script_data: Arc<SetupScriptExecuteData<'a>>,
     ) {
-        debug!(test_name = test.instance.name, "running test");
+        debug!(test_name = %test.instance.name, "running test");
 
         let settings = Arc::new(test.settings);
 
@@ -300,8 +300,9 @@ impl<'a> ExecutorContext<'a> {
                     .next()
                     .expect("backoff delay must be non-empty");
 
-                let run_status = run_status.into_external();
+                // Capture the internal result before converting to external.
                 let previous_result = run_status.result;
+                let run_status = run_status.into_external();
                 let previous_slow = run_status.is_slow;
 
                 let _ = resp_tx.send(ExecutorEvent::AttemptFailedWillRetry {
@@ -387,6 +388,7 @@ impl<'a> ExecutorContext<'a> {
         let command_mut = cmd.command_mut();
 
         command_mut.env("NEXTEST_RUN_ID", format!("{}", self.run_id));
+        command_mut.env("NEXTEST_RUN_MODE", self.test_list.mode().to_string());
         command_mut.stdin(Stdio::null());
         super::os::set_process_group(command_mut);
 
@@ -624,18 +626,20 @@ impl<'a> ExecutorContext<'a> {
             create_execution_result(exit_status, &child_acc.errors, leaked, leak_timeout.result)
         });
 
-        // Read from the environment map. If there's an error here, add it to the list of child errors.
+        // Read from the environment map. If there's an error here, add it to
+        // the list of child errors and treat the setup script as failed.
         let mut errors: Vec<_> = child_acc.errors.into_iter().map(ChildError::from).collect();
-        let env_map = if exec_result.is_success() {
+        let (env_map, exec_result) = if exec_result.is_success() {
             match parse_env_file(&env_path).await {
-                Ok(env_map) => Some(env_map),
+                Ok(env_map) => (Some(env_map), exec_result),
                 Err(error) => {
                     errors.push(ChildError::SetupScriptOutput(error));
-                    None
+                    // ExecFail isn't *quite* right but it's close enough.
+                    (None, ExecutionResult::ExecFail)
                 }
             }
         } else {
-            None
+            (None, exec_result)
         };
 
         Ok(InternalSetupScriptExecuteStatus {
@@ -703,11 +707,12 @@ impl<'a> ExecutorContext<'a> {
 
         // Test-related environment variables.
         command_mut.env("NEXTEST_RUN_ID", format!("{}", self.run_id));
+        command_mut.env("NEXTEST_RUN_MODE", self.test_list.mode().to_string());
         command_mut.env(
             "NEXTEST_BINARY_ID",
             test.test_instance.suite_info.binary_id.as_str(),
         );
-        command_mut.env("NEXTEST_TEST_NAME", test.test_instance.name);
+        command_mut.env("NEXTEST_TEST_NAME", test.test_instance.name.as_str());
         command_mut.env("NEXTEST_ATTEMPT", test.retry_data.attempt.to_string());
         command_mut.env(
             "NEXTEST_TOTAL_ATTEMPTS",
@@ -1235,7 +1240,7 @@ impl<'a> TestPacket<'a> {
             test_instance: self.test_instance.id(),
             state,
             retry_data: self.retry_data,
-            output,
+            output: output.into(),
         })
     }
 }
@@ -1288,9 +1293,9 @@ impl<'a> SetupScriptPacket<'a> {
             stress_index: self.stress_index,
             script_id: self.script_id.clone(),
             program: self.program.clone(),
-            args: &self.config.command.args,
+            args: self.config.command.args.clone(),
             state,
-            output,
+            output: output.into(),
         })
     }
 }
